@@ -1,4 +1,4 @@
-from os import chdir
+from os import chdir, getcwd
 from os.path import dirname, basename
 from unittest import TestCase
 
@@ -12,4 +12,4 @@ class UnitTests(TestCase):
 
     def test_human_path(self):
         chdir(dirname(__file__))
-        eq_(human_path(__file__), basename(__file__))
+        eq_(human_path(__file__, getcwd()), basename(__file__))
